import { EventEmitter } from 'events';
import { CreateBaseSummarizerParams, SummarizerEventEmitter } from '../../common';
import {
  retrieveVideoId,
  getVideoPage,
  extractDataFromPage,
  fetchTranscriptItems,
} from './transcript';
import { normalizeTranscriptText, summarizeTranscript } from '../../summarizer';

export interface CreateYouTubeSummarizerParams extends CreateBaseSummarizerParams {}

export class YouTubeSummarizerEventEmitter extends EventEmitter implements SummarizerEventEmitter {
  constructor(private readonly params: CreateYouTubeSummarizerParams) {
    super();
  }

  process() {
    const {
      url,
      openaiApiKey,
      openaiOrganizationId,
      ...config
    } = this.params;
    const identifier = retrieveVideoId(url);

    this.emit('process', {
      processType: 'extract-data',
      phase: 'download-page',
    });

    getVideoPage(identifier)
      .then((videoPageBody) => {
        const pageData = extractDataFromPage(videoPageBody);
        this.emit('process', {
          processType: 'extract-data',
          phase: 'success',
        });

        this.emit('process', {
          processType: 'fetch-transcript',
          phase: 'start',
        });
        return fetchTranscriptItems(pageData, config);
      })
      .then((transcript) => {
        this.emit('process', {
          processType: 'fetch-transcript',
          phase: 'success',
          content: JSON.stringify(transcript),
          contentType: 'application/json',
        });

        this.emit('process', {
          processType: 'normalize-caption',
          phase: 'start',
        });

        return normalizeTranscriptText(
          transcript.map((item) => item.text).join(' '),
          openaiApiKey,
          openaiOrganizationId,
        );
      })
      .then((normalizedCaption) => {
        this.emit('process', {
          processType: 'normalize-transcript',
          phase: 'success',
          content: normalizedCaption,
          contentType: 'text/plain',
        });

        this.emit('process', {
          processType: 'summarize-transcript',
          phase: 'start',
        });

        return summarizeTranscript(normalizedCaption, openaiApiKey, openaiOrganizationId);
      })
      .then((summary) => {
        this.emit('process', {
          processType: 'summarize-transcript',
          phase: 'success',
          data: summary,
          contentType: 'text/plain',
        });

        this.emit('end');
      })
      .catch((error) => {
        this.emit('error', error);
        this.emit('end');
      });
  }
}
